//
//  SFController.m
//  StuffItNameFix
//
//  Created by Chang-Beom Park on 2/19/10.
//  Copyright 2010 trip2me.pe.kr. All rights reserved.
//

#import "SFController.h"
#import "SFConverter.h"
#import "SFView.h"

NSString *SFConverterType             = @"Convert type";
NSString *SFIncludeDirectory          = @"Include sub-directory";
NSString *SFTryMacKorean              = @"Try to convert MacKorean";


@implementation SFController

+ (void)initialize
{
	NSUserDefaults *ud;
	NSMutableDictionary *defaults;
	
    static BOOL initialized = NO;
    /* Make sure code only gets executed once. */
    if (initialized == YES) return;
    initialized = YES;
    
	ud = [NSUserDefaults standardUserDefaults];
	defaults = [NSMutableDictionary dictionary];
    
	[defaults setObject:[NSNumber numberWithInt:OS9toOSX] forKey:SFConverterType];
    [defaults setObject:[NSNumber numberWithBool:YES] forKey:SFIncludeDirectory];
    [defaults setObject:[NSNumber numberWithBool:NO] forKey:SFTryMacKorean];

	[ud registerDefaults:defaults];
}

- (id) init {
    self = [super init];
    return self;
}

- (void)awakeFromNib
{
	NSUserDefaults *ud;
    
    ud = [NSUserDefaults standardUserDefaults];
    
    [convertType selectItemAtIndex:[[ud objectForKey:SFConverterType] intValue]];
    [directoryInclude setState:[[ud objectForKey:SFIncludeDirectory] boolValue]];
    [tryMacKoreanCheckBox setState:[[ud objectForKey:SFTryMacKorean] boolValue]];
    
    converterType = [[ud objectForKey:SFConverterType] intValue];
    includeSubDirectory = [[ud objectForKey:SFIncludeDirectory] boolValue];
    tryMacKorean = [[ud objectForKey:SFTryMacKorean] boolValue];
    
    NSNotificationCenter *nc;    
	nc = [NSNotificationCenter defaultCenter];
    
	[nc addObserver:self selector:@selector(filesDropped:)
               name:SFFilesDroppedNotification object:nil];
}

- (void)applicationWillFinishLaunching:(NSNotification *)n
{
	terminateAfterConverting = -1;
}

- (void)applicationDidFinishLaunching:(NSNotification *)n
{    
	if (terminateAfterConverting == -1)
		terminateAfterConverting = NO;
    
    [NSApp setServicesProvider:self];
}

- (BOOL)windowShouldClose:(id)window
{
    if (terminateAfterConverting == YES )
        return YES;
    else
    {
        [NSApp hide:self];
        return NO;
    }
}


- (void)windowWillClose:(NSNotification *)n
{
    NSNotificationCenter *nc;
    
	nc = [NSNotificationCenter defaultCenter];

    [NSApp terminate:self];    
	[nc removeObserver:self];

    [self release];
}

- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames
{
    if (terminateAfterConverting == -1)
		terminateAfterConverting = YES;
    
    [self prepare:filenames];
}

- (IBAction)converterChanged:(id)sender
{
    converterType = [convertType indexOfSelectedItem];
    
    NSUserDefaults *ud;
    
    ud = [NSUserDefaults standardUserDefaults];
    
	[ud setObject:[NSNumber numberWithInt:converterType] forKey:SFConverterType];    
}

- (IBAction)includeDirectoryChanged:(id)sender
{
    includeSubDirectory = [directoryInclude state];
    
    NSUserDefaults *ud;
    
    ud = [NSUserDefaults standardUserDefaults];
    
	[ud setBool:includeSubDirectory forKey:SFIncludeDirectory];
}

- (IBAction)tryMacKoreanChanged:(id)sender
{
    tryMacKorean = [tryMacKoreanCheckBox state];
    
    NSUserDefaults *ud;
    
    ud = [NSUserDefaults standardUserDefaults];
    
	[ud setBool:tryMacKorean forKey:SFTryMacKorean];
}


- (void)filesDropped:(NSNotification *)n
{    
    [self prepare:[n object]];
}

- (void)paste:(id)sender
{
    NSPasteboard *pasteboard = [NSPasteboard generalPasteboard];

    if([[pasteboard types] containsObject:NSFilenamesPboardType])
    {
        NSArray *filenames = [pasteboard propertyListForType:NSFilenamesPboardType];
        [self prepare:filenames];
    }
}   

- (void)prepare:(NSArray *)filenames
{
    NSAutoreleasePool    *autoreleasepool = [[NSAutoreleasePool alloc] init];

    [spinner setHidden:NO];
    [spinner startAnimation:self];
    
    converter = [[SFConverter alloc] init];
    if ( converterType == OS9toOSX )
    {
        int i;
        for ( i = 0; i < [filenames count]; i++)
        {
            NSString* curDir = [[filenames objectAtIndex:i] stringByDeletingLastPathComponent];
            NSArray* filename = [NSArray arrayWithObject:[[filenames objectAtIndex:i] lastPathComponent]];
            [converter convertOS9Filename:filename currentDirectory:curDir includeDirectory:includeSubDirectory tryMacKorean:tryMacKorean];
        }
    }
    else
    {
        int i;
        for ( i = 0; i < [filenames count]; i++)
        {
            NSString* curDir = [[filenames objectAtIndex:i] stringByDeletingLastPathComponent];
            NSArray* filename = [NSArray arrayWithObject:[[filenames objectAtIndex:i] lastPathComponent]];
            [converter convertOSXFilename:filename currentDirectory:curDir includeDirectory:includeSubDirectory tryMacKorean:tryMacKorean];
        }
    }
    [converter release];
    
    [spinner stopAnimation:self];
    [spinner setHidden:YES];

    [autoreleasepool release];
    
    if (terminateAfterConverting == YES)
		[NSApp terminate:self];
    
}

- (void)OSXFilenameFixWithPasteboard:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
    if([[pboard types] containsObject:NSFilenamesPboardType])
    {
        NSArray *filenames=[pboard propertyListForType:NSFilenamesPboardType];
        terminateAfterConverting = YES;
        
        converterType = OSXtoOS9;
        [self prepare:filenames];
    }
}

- (void)OS9FilenameFixWithPasteboard:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
    if([[pboard types] containsObject:NSFilenamesPboardType])
    {
        NSArray *filenames=[pboard propertyListForType:NSFilenamesPboardType];
        terminateAfterConverting = YES;
        
        converterType = OS9toOSX;
        [self prepare:filenames];
    }
}

@end
