//
//  SFConverter.m
//  StuffItNameFix
//
//  Created by Chang-Beom Park on 2/20/10.
//  Copyright 2010 trip2me.pe.kr. All rights reserved.
//

#import "SFConverter.h"
#include "StuffItKoreanConvertTable.h"


@implementation SFConverter

- (id)init
{
	self = [super init];
    fm = [NSFileManager defaultManager];
	return self;
}

- (void)dealloc
{
	[super dealloc];
}


- (void)convertOS9Filename:(NSArray *)filenames currentDirectory:(NSString *)curDir includeDirectory:(BOOL)isRecursive tryMacKorean:(BOOL)tryMacKorean
{
    NSAutoreleasePool    *autoreleasepool = [[NSAutoreleasePool alloc] init]; 
    int count;
    bool isLatinConverted = false;

    
    [fm changeCurrentDirectoryPath:curDir];
    //NSLog(curDir);

    int fileCount = [filenames count];
    for (count = 0; count < fileCount; count++)
    {
        NSString* originalFilename = [filenames objectAtIndex:count];
        //NSLog([NSString stringWithFormat:@"%d %@",[originalFilename length],originalFilename]);
        
        BOOL isDir;
        NSString * newDir = [curDir stringByAppendingPathComponent:originalFilename];
        [fm fileExistsAtPath:newDir isDirectory:&isDir];
        
        if (isDir && isRecursive)
        {
            NSArray* fileListInDir = [fm directoryContentsAtPath:newDir];
            [self convertOS9Filename:fileListInDir currentDirectory:newDir includeDirectory:isRecursive tryMacKorean:tryMacKorean];
            [fm changeCurrentDirectoryPath:curDir];
        }

        int stringLength = [originalFilename length];
        [originalFilename getCharacters:buffer];
        
        int index;
        for (index = 0; index < stringLength; index++)
        {
            //NSLog([NSString stringWithFormat:@"%x",buffer[index]]);

            if (buffer[index] > 0x33FF && buffer[index] < 0x3D2E)
                buffer[index] = StuffItKorean_to_Unicode[buffer[index]-0x3400];            
            if (buffer[index] > 0x84 && buffer[index] < 0xA0)
            {
                buffer[index] = StuffItLatin_to_Unicode[buffer[index]-0x80];
                isLatinConverted = true;
            }
            //NSLog([NSString stringWithFormat:@"%x",buffer[index]]);
        }
        
        NSString * newFilename;
        newFilename = [NSString stringWithCharacters:buffer length:stringLength];
        
        if ( tryMacKorean && !isLatinConverted &&[newFilename canBeConvertedToEncoding:NSMacOSRomanStringEncoding] )
        {
            NSData *convertedData = [newFilename dataUsingEncoding:NSMacOSRomanStringEncoding];
            //NSLog([NSString stringWithFormat:@"%d : %@",[convertedData length],[convertedData description]]);

            newFilename = [[NSString alloc] initWithData:convertedData encoding:0x80000003];
            //NSLog([NSString stringWithFormat:@"%d : %@",[newFilename length],newFilename]);
        }
        
        if ( [newFilename compare:originalFilename] )
        {            
            [fm movePath:[filenames objectAtIndex:count] toPath:newFilename handler:nil];
            [[NSWorkspace sharedWorkspace] noteFileSystemChanged:newFilename];
        }
        isLatinConverted = false;
    }
    [autoreleasepool release];
}


- (void)convertOSXFilename:(NSArray *)filenames currentDirectory:(NSString *)curDir includeDirectory:(BOOL)isRecursive tryMacKorean:(BOOL)tryMacKorean
{
    NSAutoreleasePool    *autoreleasepool = [[NSAutoreleasePool alloc] init]; 

    int count;
    
    [fm changeCurrentDirectoryPath:curDir];
    //NSLog(curDir);

    int fileCount = [filenames count];
    for (count = 0; count < fileCount; count++)
    {
        NSString* originalFilename = [[filenames objectAtIndex:count] precomposedStringWithCanonicalMapping];
        //NSLog([NSString stringWithFormat:@"%d %@",[originalFilename length],originalFilename]);
        
        BOOL isDir;
        NSString * newDir = [curDir stringByAppendingPathComponent:originalFilename];
        [fm fileExistsAtPath:newDir isDirectory:&isDir];
        
        if (isDir && isRecursive)
        {
            NSArray* fileListInDir = [fm directoryContentsAtPath:newDir];
            [self convertOSXFilename:fileListInDir currentDirectory:newDir includeDirectory:isRecursive tryMacKorean:tryMacKorean];
            [fm changeCurrentDirectoryPath:curDir];
        }
        
        int stringLength = [originalFilename length];
        [originalFilename getCharacters:buffer];
        
        int index;
        for (index = 0; index < stringLength; index++)
        {
            if (buffer[index] > 0xABFF && buffer[index] < 0xD7A4)
                buffer[index] = Unicode_to_StuffItKorean[buffer[index]-0xAC00];
            if (buffer[index] > 0xCF && buffer[index] < 0x100)
                if ( Unicode_to_StuffItLatin[buffer[index]-0xD0] != 0)
                    buffer[index] = Unicode_to_StuffItLatin[buffer[index]-0xD0];
            //NSLog([NSString stringWithFormat:@"%x",buffer[index]]);
        }
        
        NSString * newFilename;
        newFilename = [NSString stringWithCharacters:buffer length:stringLength];

        // FIXME: Conversion case has not implemented completely.
        if ( tryMacKorean && [newFilename canBeConvertedToEncoding:0x80000003] )
        {
            NSData *convertedData = [newFilename dataUsingEncoding:0x80000003];
            //NSLog([NSString stringWithFormat:@"%d : %@",[convertedData length],[convertedData description]]);
            
            newFilename = [[NSString alloc] initWithData:convertedData encoding:NSMacOSRomanStringEncoding];
            //NSLog([NSString stringWithFormat:@"%d : %@",[newFilename length],newFilename]);
        }
        
        if ( [newFilename compare:originalFilename] )
        {
            if ( [fm movePath:[filenames objectAtIndex:count] toPath:newFilename handler:nil] == YES )
            {
                [[NSWorkspace sharedWorkspace] noteFileSystemChanged:newFilename];
            }
            else
            {
                NSLog([NSLocalizedString(@"Failed to rename file :",nil) stringByAppendingString:[filenames objectAtIndex:count]]);
            }
        }
    }
    [autoreleasepool release];
}

@end
